@@ -72,6 +72,7 @@ class Environment(object):
     provides an execution environment for jobs
     '''
     def __init__(self, doc=None, version='', handle_files=True):
+        self._target_dir = None
         self._script_version = version
         self._unprocessed_sequence = []
         self._sequence = []
@@ -144,8 +145,7 @@ class Environment(object):
         if self._is_first_time:
             self._update_sequence(**kwargs)
             self._is_first_time = False
-        sequence = self._sequence
-        return sequence
+        return self._sequence
 
     def _update_sequence(self, files=[], recursive=False, **kwargs):
         '''
@@ -195,17 +195,14 @@ class Environment(object):
         '''
         returns the class being used as the filename parser
         
-        if with_args is True, then partial is used to apply arguments as
+        if more kwargs are supplied, then partial is used to apply arguments as
         appropriate
         '''
-        if kwargs is not None:
-            debug('Using %s with the following kwargs:',
-                  self._filename_parser.__name__)
-            debug('\n%s', pprint.pformat(kwargs))
-            return partial(self._filename_parser, **kwargs)
-        else:
-            debug('Using %s', self._filename_parser.__name__)
-            return self._filename_parser
+        debug('Using %s with the following kwargs:',
+              self._filename_parser.__name__)
+        kwargs['target_dir'] = self.get_target_dir()
+        debug('\n%s', pprint.pformat(kwargs))
+        return partial(self._filename_parser, **kwargs)
 
     def override_num_cpus(self, num):
         """
@@ -244,6 +241,20 @@ class Environment(object):
                 debug('Skipping %s because file does not have a valid file '
                       'extension', _quote(f))
                 return False
+   
+    def get_context(self):
+        args = self.argument_parser.parse_args()
+        context = vars(args)
+        context['target_dir'] = self.get_target_dir(context['target'])
+
+        # read config if user supplies method
+        if context['config'] is not None:
+            if self._config_reader is None: raise NotImplementedError ### FINISH
+            cfg_opts = self._config_reader(context['config'])
+            context.update(cfg_opts)
+
+        return context
+
     
     @exit_on_Usage 
     def do_action(self, action, stay_open=False):
@@ -252,15 +263,7 @@ class Environment(object):
         
         actions should be functions that at least take FilenameParser objects
         '''
-        args = self.argument_parser.parse_args()
-        context = vars(args)
-        # read config if user supplies method
-        if context['config'] is not None:
-            if self._config_reader is None: raise NotImplementedError ### FINISH
-            cfg_opts = self._config_reader(context['config'])
-            context.update(cfg_opts)
-        
-        self._options = context
+        context = self.get_context()
         LOGGER.setLevel(context['logging_level'])
         
         num_cpus = self._num_cpus or context['num_cpus']
@@ -278,9 +281,9 @@ class Environment(object):
         
         max_cpus = len(sequence)
         debug('Debugging mode enabled')
-    
+         
         used_cpus = min([num_cpus, max_cpus])
-    
+   
         # write config if user supplies method
         if self._config_writer is not None:
             self._config_writer(**context)
@@ -318,6 +321,24 @@ class Environment(object):
         '''
         raise NotImplementedError
         os.execlp(self.next_script, "--find")
+
+    def get_target_dir(self, name=None):
+        if self._target_dir is None: self._target_dir = self._construct_target(name)
+        return self._target_dir
+
+    def _construct_target(self, name=None):
+        if name is None: name = PROGRAM_NAME
+        try: name = name[0:name.index('.py')]
+        except ValueError: pass
+        date = time.strftime("%m-%d-%Y", time.localtime())
+        user = getpass.getuser()
+        t = '_'.join([name, date, user])
+        i = 0
+        while True:
+            target = '%s.%s' % (t, i)
+            if os.path.exists(target): i += 1
+            else: break
+        return target
     
 def _quote(s):
     return ''.join(["'", s ,"'"])
@@ -336,19 +357,6 @@ def assert_path(path):
     else:
         raise IOError(ENOENT, os.strerror(ENOENT), path)
 
-def construct_target(name=None):
-    if name is None: name = PROGRAM_NAME
-    try: name = name[0:name.index('.py')]
-    except ValueError: pass
-    date = time.strftime("%m-%d-%Y", time.localtime())
-    user = getpass.getuser()
-    t = '_'.join([name, date, user])
-    i = 0
-    while True:
-        target = '%s.%s' % (t, i)
-        if os.path.exists(target): i += 1
-        else: break
-    return target
 
 class FilenameParser(object):
     """
@@ -361,7 +369,7 @@ class FilenameParser(object):
     """
     @exit_on_Usage
     def __init__(self, filename, drop_parent_name=True,
-                 target=None, no_target=False, *args, **kwargs):
+                 target_dir=None, no_target=False, *args, **kwargs):
         self.additional_args = args
         self.__dict__.update(kwargs)
 
@@ -376,8 +384,10 @@ class FilenameParser(object):
 
         if no_target:
             output_dir = '.'
+        elif target_dir is None:
+            warning('Something went wrong setting the target directory. Using current directory instead')
+            output_dir = '.'
         else:
-            target_dir = construct_target(target)
             if drop_parent_name:
                 path_by_folder = input_dir.split(os.sep)
                 if len(path_by_folder) == 1:
